// storm
#ifdef OPENGL_ES
#extension GL_OES_standard_derivatives : enable
#ifdef GL_FRAGMENT_PRECISION_HIGH
precision highp float;
#else
precision mediump float;
#endif
#else
precision mediump float;
#endif

// Uniforms
uniform sampler2D u_texture;
uniform float u_time;
uniform vec3 u_resolution;

// Varyings
varying vec2 v_texCoord;

#define PI 3.141592

float cosineInterpolation(float a, float b, float x)
{
	float ft = x * PI;
	float f = (1.0 - cos(ft)) * 0.5;
	return a * (1.0 - f) + b * f;
}

float noise11(float p)
{
	return fract(sin(p * 633.1847) * 9827.95);
}

float smoothNoise11(float p, float dist)
{
	float prev = noise11(p - dist);
	float next = noise11(p + dist);

	return cosineInterpolation(prev, next, 0.5);
}

float valueNoise11(float p)
{
	float c = smoothNoise11(p, 0.5);
	c += smoothNoise11(p, 0.25) * 0.5;
	c += smoothNoise11(p, 0.125) * 0.25;
	c += smoothNoise11(p, 0.0625) * 0.125;

	return c /= 0.875;
}

float noise21(vec2 p)
{
	return fract(sin(p.x * 827.221 + p.y * 3228.8275) * 878.121);
}

float smoothNoise21(vec2 uv, float cells)
{
	vec2 lv = fract(uv * cells);
	vec2 id = floor(uv * cells);

	//smoothstep function: maybe change it later!
	lv = lv * lv * (3.0 - 2.0 * lv);

	float bl = noise21(id);
	float br = noise21(id + vec2(1.0, 0.0));
	float b = mix(bl, br, lv.x);

	float tl = noise21(id+vec2(0.0, 1.0));
	float tr = noise21(id+vec2(1.0, 1.0));
	float t = mix(tl, tr, lv.x);

	return mix(b, t, lv.y);
}

float valueNoise21(vec2 uv)
{
	float c = smoothNoise21(uv, 4.0);
	c += smoothNoise21(uv, 8.0) * 0.5;
	c += smoothNoise21(uv, 16.0) * 0.25;
	c += smoothNoise21(uv, 32.0) * 0.125;
	c += smoothNoise21(uv, 64.0) * 0.0625;

	return c /= 0.9375;
}

void main()
{
	vec2 uv = gl_FragCoord.xy / u_resolution.xy;
	float time = u_time * 20.0;

	vec3 c = vec3(0.15, 0.15, 0.02);
	c *= 0.65 + (1.0 - uv.y);

	uv.x += time * 0.1;
	uv.y += valueNoise11(uv.x + 0.352) * 0.01;
	c += valueNoise21(uv) * 0.1;

	// Output to screen
	vec4 t = texture2D(u_texture, v_texCoord);
	t += clamp(vec4(c, 1.0), 0.0, 1.0);
	gl_FragColor = vec4(t.rgb, 1.0);
}
